import pygame, sys
import Image as Image
import copy


"""
Tool for selecting a set of rectangles in an image.
setup() must be called first, with a path to the image file.
Then selectRects can be called.

Usage:
>> pygame.init()
>> path = "img.jpg"
>> screen, px = setup(path)
>> rects = selectRects(screen, px, 4)
>> len(rects)
4
>>>
>>> pygame.display.quit()
"""



#-----------------------------------------------------------------
"""
Displays the image on screen, draws the previously done rectangles
and the currently active rectangle.
"""
#-----------------------------------------------------------------

def displayImage(screen, px, topleft, prior, done_rects):
    # ensure that the rect always has positive width, height
    x, y = topleft
    width =  pygame.mouse.get_pos()[0] - topleft[0]
    height = pygame.mouse.get_pos()[1] - topleft[1]
    if width < 0:
        x += width
        width = abs(width)
    if height < 0:
        y += height
        height = abs(height)

    # eliminate redundant drawing cycles (when mouse isn't moving)
    current = x, y, width, height
    if not (width and height):
        return current
    if current == prior:
        return current

    # draw transparent box and blit it onto canvas
    screen.blit(px, px.get_rect())

    # draw the done rects on the image
    for rect in done_rects:
        w = int(rect[2] - rect[0])
        h = int(rect[3] - rect[1])
        im_done = pygame.Surface((w, h))
        im_done.fill((0, 0, 128))
        pygame.draw.rect(im_done, (32, 32, 32), im_done.get_rect(), 1)
        im_done.set_alpha(100)#128
        screen.blit(im_done, (rect[0], rect[1]))
        pygame.display.flip()

    im = pygame.Surface((width, height))
    im.fill((128, 128, 128))
    pygame.draw.rect(im, (32, 32, 32), im.get_rect(), 1)
    im.set_alpha(100)#128
    screen.blit(im, (x, y))
    pygame.display.flip()

    # return current box extents
    return (x, y, width, height)

#-----------------------------------------------------------------
"""
Creates an inital pygame surface and draws the image from path
on it.
path - the path to the image file
im_size (optional) - size to display on screen
screen = the display surface
px = a surface with the image on it
"""
#-----------------------------------------------------------------
def setup(path, im_size=None):
	pygame.display.set_caption
	px = pygame.image.load(path)
	if im_size:
		px = pygame.transform.scale(px, im_size)
	screen = pygame.display.set_mode( px.get_rect()[2:] )
	#screen = pygame.display.set_mode( size ) #, pygame.FULLSCREEN,
	screen.blit(px, px.get_rect())
	screen
	pygame.display.flip()
	return screen, px

#-----------------------------------------------------------------
"""
Fixes a case in which the rect is define as (bottomright, topleft)
instead of (topleft, bottomright)
"""
#-----------------------------------------------------------------
def fix_rect(rect):
	left, upper, right, lower = rect
    # Ensure output rect always has positive width, height

	if right < left:
	    left, right = right, left
	if lower < upper:
	    lower, upper = upper, lower

	return (left, upper, right, lower)


def displayCleanImage(screen, px, done_rects):
	screen.blit(px, px.get_rect())
	pygame.display.flip()
	# draw the done rects on the image
	for rect in done_rects:
		w = int(rect[2] - rect[0])
		h = int(rect[3] - rect[1])
		im_done = pygame.Surface((w, h))
		im_done.fill((0, 0, 128))
		pygame.draw.rect(im_done, (32, 32, 32), im_done.get_rect(), 1)
		im_done.set_alpha(128)
		screen.blit(im_done, (rect[0], rect[1]))
		pygame.display.flip()


#-----------------------------------------------------------------
"""
Selects num_rects rectangles in the image.
Must call setup once before calling this function.
"""
#-----------------------------------------------------------------
def selectRects(screen, px, num_rects, done_rects=[]):
    tmp_rects = copy.deepcopy(done_rects)
    rects = []
    for i in range(num_rects):
        topleft = bottomright = prior = None
        n=0
        while n!=1:
            for event in pygame.event.get():
                if event.type == pygame.MOUSEBUTTONUP:
                    if not topleft:
                        topleft = event.pos
                    else:
                        bottomright = event.pos
                        n=1
            if topleft:
                prior = displayImage(screen, px, topleft, prior, tmp_rects)


        tmp_rects.append(fix_rect((topleft + bottomright)))
        rects.append(fix_rect((topleft + bottomright)))
        # draw rect last time, now in done color
        prior = displayImage(screen, px, topleft, [], tmp_rects)

    return rects
